﻿using Infrastructure;
using Infrastructure.Prism;
using Microsoft.Practices.Prism;
using Microsoft.Practices.Prism.Commands;
using System;
using System.ComponentModel;

namespace ModuleA
{
    public class ViewAViewModel : INotifyPropertyChanged, IActiveAware, IDockAware
    {
        public static Random _randomNumberGenerator = new Random();
        int _randomMaxValue = 289876756;

        public ViewAViewModel()
        {
            Number = _randomNumberGenerator.Next(_randomMaxValue);

            UpdateCommand = new DelegateCommand(Update);
            KnownCommands.UpdateViewCommand.RegisterCommand(UpdateCommand);
        }

        private int _number;
        public int Number
        {
            get { return _number; }
            set
            {
                _number = value;
                Header = String.Format("Title: {0}", _number);
                NotifyPropertyChanged("Number");
            }
        }

        public DelegateCommand UpdateCommand { get; set; }

        void Update()
        {
            Number = _randomNumberGenerator.Next(_randomMaxValue);
        }

        #region IActiveAware

        bool _isActive;
        public bool IsActive
        {
            get { return _isActive; }
            set
            {
                _isActive = value;
                OnIsActiveChanged();
            }
        }

        public event EventHandler IsActiveChanged;

        protected virtual void OnIsActiveChanged()
        {
            UpdateCommand.IsActive = _isActive;

            NotifyPropertyChanged("IsActive");

            if (IsActiveChanged != null)
                IsActiveChanged(this, EventArgs.Empty);
        }

        #endregion //IActiveAware

        #region IDockAware

        private string _header;
        public string Header
        {
            get { return _header; }
            set
            {
                _header = value;
                NotifyPropertyChanged("Header");
            }
        }

        #endregion //IDockAware

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion //INotifyPropertyChanged
    }
}
