﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interop;

namespace XamDockManagerPrismRegionAdapter
{
    /// <summary>
    /// Interaction logic for Shell.xaml
    /// </summary>
    public partial class Shell : Window
    {
        public Shell(ShellViewModel viewModel)
        {
            InitializeComponent();
            DataContext = viewModel;
        }        

        protected override void OnSourceInitialized(EventArgs e)
        {
            base.OnSourceInitialized(e);

            var source = HwndSource.FromVisual(this) as HwndSource;
            source.AddHook(FilterMessage);
        }

        /// <summary>
        /// Windows message loop filter. 
        /// Watches for WM_MOUSEACTIVATE and returns MA_NOACTIVATE for the main window.
        /// This prevents the main window from getting focus due to a menu click.
        /// </summary>
        private IntPtr FilterMessage(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            const int WM_MOUSEACTIVATE = 0x0021;
            const int MA_NOACTIVATE = 3;

            switch (msg)
            {
                case WM_MOUSEACTIVATE:
                    handled = true;
                    // TELL WINDOWS NOT TO ACTIVATE MAIN WINDOW.
                    return new IntPtr(MA_NOACTIVATE);
            }
            return IntPtr.Zero;
        }

        private void XamRibbon_PreviewGotKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
        {
            //prevent the XamRibbon from gaining keyboard focus
            e.Handled = true;
        }
    }
}
