﻿using Infrastructure;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.Unity;
using ModuleA;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace XamDockManagerPrismRegionAdapter
{
    public class ShellViewModel : INotifyPropertyChanged
    {
        IRegionManager _regionManager;
        IUnityContainer _container;

        private ObservableCollection<ViewAViewModel> _views;
        public ObservableCollection<ViewAViewModel> Views
        {
            get { return _views; }
            set
            {
                _views = value;
                NotifyPropertyChanged("Views");
            }
        }

        public DelegateCommand<object> ActivateCommand { get; set; }
        public DelegateCommand<object> OpenCommand { get; set; }
        public DelegateCommand<object> CloseCommand { get; set; }

        public ShellViewModel(IUnityContainer container, IRegionManager regionManager)
        {
            _container = container;
            _regionManager = regionManager;

            ActivateCommand = new DelegateCommand<object>(Activate);
            OpenCommand = new DelegateCommand<object>(Open, CanOpen);
            CloseCommand = new DelegateCommand<object>(Close);

            //generate some data
            Views = new ObservableCollection<ViewAViewModel>();
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
            Views.Add(_container.Resolve<ViewAViewModel>());
        }

        void Activate(object viewModel)
        {
            if (viewModel == null)
                return;

            IRegion region = _regionManager.Regions[KnownRegionNames.TabGroupPaneOne]; //get the region

            var viewToActivite = GetViewFromRegionFromViewModel(viewModel, region);
            if (viewToActivite != null)
                region.Activate(viewToActivite); //active the view
        }

        void Close(object viewModel)
        {
            if (viewModel == null)
                return;

            IRegion region = _regionManager.Regions[KnownRegionNames.TabGroupPaneOne]; //get the region

            var viewToRemove = GetViewFromRegionFromViewModel(viewModel, region);
            if (viewToRemove != null)
                region.Remove(viewToRemove); //active the view
        }

        void Open(object viewModel)
        {
            IRegion region = _regionManager.Regions[KnownRegionNames.TabGroupPaneOne]; //get the region

            var view = _container.Resolve<ViewA>(); //create the view to use inside the content pane of the dock manager
            view.DataContext = viewModel; //set the data context

            region.Add(view); //add the view
            region.Activate(view); //active the view  
        }

        bool CanOpen(object viewModel)
        {
            return viewModel != null;
        }

        object GetViewFromRegionFromViewModel(object viewModel, IRegion region)
        {
            foreach (FrameworkElement view in region.Views)
            {
                if (view.DataContext == viewModel)
                {
                    return view;
                }
            }

            return null;
        }

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion //INotifyPropertyChanged
    }
}
